package com.astrum.utils;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.nio.Buffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;

/**
 * Created by ebicer on 9/5/16.
 */
public class HttpCore {
    public interface ResponseListener<T> {
        void onResponse(T response, HashMap<String, List<String>> headers);

        void onError(int statusCode, HashMap<String, List<String>> headers);
    }

    public static <T> void postJSONAsync(final String url, final T requestData, final HashMap<String, String> headers, final Type responseType, final ResponseListener listener) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                postJSON(url, requestData, headers, responseType, listener);

            }
        }).start();
    }

    public static <T> Object postJSON(String url, T requestData, HashMap<String, String> headers, Type responseType, ResponseListener listener) {
        Gson gson = new GsonBuilder().create();
        String json;
        if (!(requestData instanceof String)) {
            json = gson.toJson(requestData);
        } else {
            json = requestData.toString();
        }
        HashMap<String, List<String>> responseHeaders = new HashMap<>();
        try {
            byte[] result = post(url, json, headers, responseHeaders);
            if (result != null) {
                json = new String(result, "utf-8");
                Object response = json;
                if (responseType != String.class) {
                    response = gson.fromJson(json, responseType);
                }
                if (listener != null) {
                    listener.onResponse(response, responseHeaders);
                }
                return response;
            } else {
                if (listener != null) {
                    listener.onResponse(null, responseHeaders);
                }
            }
        } catch (HttpStatusException ex) {
            listener.onError(ex.getStatusCode(), responseHeaders);
        } catch (Exception e) {
            listener.onError(-1, null);
        }
        return null;
    }

    public static void getJSONAsync(final String url, final HashMap<String, String> headers, final Type responseType, final ResponseListener listener) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                getJSON(url, headers, responseType, listener);
            }
        }).start();
    }

    public static Object getJSON(String url, HashMap<String, String> headers, Type responseType, ResponseListener listener) {
        Gson gson = new GsonBuilder().create();
        HashMap<String, List<String>> responseHeaders = new HashMap<>();
        try {
            byte[] result = get(url, headers, responseHeaders);
            if (result != null) {
                String json = new String(result, "utf-8");
                Object response = null;
                if (responseType == String.class) {
                    response = json;
                } else {
                    response = gson.fromJson(json, responseType);
                }
                if (listener != null) {
                    listener.onResponse(response, responseHeaders);
                }
                return response;
            } else {
                if (listener != null) {
                    listener.onResponse(null, responseHeaders);
                }
            }
        } catch (HttpStatusException ex) {
            listener.onError(ex.getStatusCode(), responseHeaders);
        } catch (Exception e) {
            listener.onError(-1, null);
        }
        return null;
    }

    public static byte[] post(String url, String requestData, HashMap<String, String> headers) throws IOException {
        return post(url, requestData, headers, null);
    }

    public static byte[] post(String url, String requestData, HashMap<String, String> headers, Map<String, List<String>> responseHeader) throws IOException {
        int result;
        URL murl = new URL(url);
        HttpURLConnection conn = (HttpURLConnection) murl.openConnection();
        conn.setRequestMethod("POST");
        conn.setUseCaches(false);
        conn.setDoInput(true);
        conn.setConnectTimeout(10000);
        conn.setReadTimeout(10000);
        conn.setDoOutput(true);
        conn.setInstanceFollowRedirects(false); // don't rely on native
        conn.setRequestProperty("Accept-Encoding", "gzip");
        conn.setRequestProperty("Accept-Charset", "UTF-8");
        conn.setRequestProperty("User-Agent", "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/31.0.1650.63 Chrome/31.0.1650.63 Safari/537.36");
        if (headers != null) {
            for (String key : headers.keySet()) {
                conn.setRequestProperty(key, headers.get(key));
            }
        }
        if (requestData != null) {
            conn.setRequestProperty("Content-Length", requestData.length() + "");
            DataOutputStream wr = new DataOutputStream(conn.getOutputStream());
            wr.writeBytes(requestData);
            wr.flush();
            wr.close();
        }
        if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
            try {
                byte[] buffer = new byte[2048];
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                InputStream reader = conn.getInputStream();
                if (conn.getHeaderField("Content-Encoding") != null) {
                    if (conn.getHeaderField("Content-Encoding").equals("gzip")) {
                        reader = new GZIPInputStream(conn.getInputStream());
                    }
                }
                do {
                    result = reader.read(buffer, 0, buffer.length);
                    if (result > 0) {
                        out.write(buffer, 0, result);
                    }
                } while (result > 0);
                if (responseHeader != null) {
                    for (String key : conn.getHeaderFields().keySet()) {
                        responseHeader.put(key, conn.getHeaderFields().get(key));
                    }
                }
                conn.disconnect();
                byte[] rst = out.toByteArray();
                out.close();
                return rst;
            } catch (HttpStatusException e) {
                conn.disconnect();
                throw e;
            } catch (SocketTimeoutException e) {
                conn.disconnect();
                throw e;
            } catch (Exception e) {
                conn.disconnect();
                throw e;
            }
        } else {
            conn.disconnect();
            throw new HttpStatusException(conn.getResponseCode(), url);
        }
    }

    public static byte[] get(String url, HashMap<String, String> headers, Map<String, List<String>> responseHeader) throws IOException {
        int result;
        URL murl = new URL(url);
        HttpURLConnection conn = (HttpURLConnection) murl.openConnection();
        conn.setRequestMethod("GET");
        conn.setUseCaches(false);
        conn.setDoInput(true);
        conn.setDoOutput(false);
        conn.setConnectTimeout(10000);
        conn.setReadTimeout(10000);
        conn.setInstanceFollowRedirects(false); // don't rely on native
        conn.setRequestProperty("Accept-Encoding", "gzip");
        conn.setRequestProperty("Accept-Charset", "UTF-8");
        conn.setRequestProperty("User-Agent", "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/31.0.1650.63 Chrome/31.0.1650.63 Safari/537.36");
        if (headers != null) {
            for (String key : headers.keySet()) {
                conn.setRequestProperty(key, headers.get(key));
            }
        }
        if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
            try {
                byte[] buffer = new byte[2048];
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                InputStream reader = conn.getInputStream();
                if ("gzip".equals(conn.getContentEncoding())) {
                    reader = new GZIPInputStream(conn.getInputStream());
                }
                do {
                    result = reader.read(buffer, 0, buffer.length);
                    if (result > 0) {
                        out.write(buffer, 0, result);
                    }
                } while (result > 0);
                if (responseHeader != null) {
                    for (String key : conn.getHeaderFields().keySet()) {
                        responseHeader.put(key, conn.getHeaderFields().get(key));
                    }
                }
                conn.disconnect();
                byte[] rst = out.toByteArray();
                out.close();
                return rst;
            } catch (HttpStatusException e) {
                conn.disconnect();
                throw e;
            } catch (SocketTimeoutException e) {
                conn.disconnect();
                throw e;
            } catch (Exception e) {
                conn.disconnect();
                throw e;
            }
        } else {
            try {
                if (responseHeader != null) {
                    for (String key : conn.getHeaderFields().keySet()) {
                        responseHeader.put(key, conn.getHeaderFields().get(key));
                    }
                }
                conn.disconnect();
            } catch (Exception e) {

            }

            throw new HttpStatusException(conn.getResponseCode(), url);
        }
    }
}
